// LCD.c
//
// Routinen zur Ansteuerung des LCD Displays
//

#include "LCD.h"

// Sendet ein Byte an das LCD
void LCD_SendByte(register unsigned char byte,register bool Data)
{
    // Abwarten bis Display bereit ist
    while (LCD_Busy());

    // erstes Nibble schicken
    PORTB = (unsigned char) ((unsigned char) byte >> 4);

    if (Data) PORTB |= _BV(RS_PORT);

    PORTB |= _BV(E_PORT);
    nop();
    nop();
    nop();
    PORTB &= ~_BV(E_PORT);

	 nop();
	 nop();
	
    // zweites Nibble schicken
    PORTB = (unsigned char) ((unsigned char) byte &0x0F);
    if (Data) PORTB |= _BV(RS_PORT);

    PORTB |= _BV(E_PORT);
    nop();
    nop();
    nop();
    PORTB &= ~_BV(E_PORT);
};

// Stellt fest, ob das LCD nicht bereit ist Daten / Befehle zu empfangen
bool LCD_Busy( void )
{
   Delayms(0.01);
	// Die Datenleitungen auf lesen stellen
    DDRB = 0xF0;

    // Befehl zum Einlesen des LCD_Status
	 PORTB = 0xF | BV(RW_PORT);
    //PORTB &= ~_BV(RS_PORT);
    //PORTB |= _;

    // Kommando senden
    PORTB |= _BV(E_PORT);
    nop();
    nop();
    nop();
	
    PORTB &= ~_BV(E_PORT);
    nop();
    nop();
    nop();

    // Ergebnis einlesen
    bool Result = PINB & _BV(D7_PORT);

	 // zweites Nibble einlesen
    PORTB |= _BV(E_PORT);
    nop();
    nop();
    nop();
	
    PORTB &= ~_BV(E_PORT);
    nop();
    nop();
    nop();

    DDRB = 0xFF;
    return Result;
};

// Initialisiert das LCD Display
void LCD_Init( void )
{
	 DDRB = 0xFF;
    register char i;
    Delayms(15.0);

    // Befehl muss drei mal wiederholt werden
    for (i = 0; i < 1; i++)
    {
        // Init Befehl
        PORTB = 0x2;

        // Kommando senden
        PORTB |= _BV(E_PORT);
        nop();
        nop();
        nop();
        PORTB &= ~_BV(E_PORT);
        nop();

        //zwischen Wiederholungen warten
        Delayms(5.0);
    };

    // Display Einstellungen vornehmen
    LCD_SendByte(LCD_INIT,false);
    LCD_SendByte(LCD_INIT,false);
    LCD_SendByte(LCD_INIT,false);
    LCD_SendByte(LCD_ON,false);
};

// Zeigt den bergebenen Text auf dem Display an
void LCD_ShowText(char Text[32])
{
	unsigned char i,j;
	LCD_SendByte(LCD_START,false);
	
	for (i = 0; i < 32; i++)
	{
		if (Text[i] == '\n')
		{
			for (j = i; j < 16; j++)
				LCD_SendByte(' ',true);
			LCD_SendByte(LCD_2NDLINE,false);
		}
   	else if (Text[i] == 0)
			for (; i < 32; i++)
				LCD_SendByte(' ',true);
		else
			LCD_SendByte(Text[i],true);
	};
};

//Schaltet die Hintergrundbeleuchtung des Displays an
void LCD_BacklightON(unsigned char Intensity)
{
	// Einstellungen zur PWM
	TCCR2 = _BV(WGM21) | _BV(WGM20) | _BV(COM21) | _BV(CS20);
	
	// Zhlen bei 0 starten
	TCNT2 = 0;
	
	// Helligkeit in Zhlervergleichsregister schreiben
	OCR2 = Intensity;
	
	// Beleuchtung einschalten
	PORTD |= _BV(PD7);
};
//Schaltet die Hintergrundbeleuchtung des Displays aus
void LCD_BacklightOFF( void )
{
	// Counter deaktivieren
	TCCR2 = 0;
	
	// Port deaktivieren
	PORTD &= ~_BV(PD7);
};








