/* clock.c
 *
 *  Sourcecode fr meine Uhr (Hauptdatei)
 *
 */

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include "lcd.h"
#include "led.h"
#include "stdio.h"
#include "string.h"
#include "thermo.h"
#include "radio.h"

// Prototypen
void Display_Aktualisieren( void );

void ShowMenu( void );
void EnableMenu(struct Menu* MenuPtr);
void DisableMenu( void );
void MenuEintrag( char* Text, void *Data);
void MenuEnableCallback ( void );
void MenuDisableCallback ( void );
void MenuPfeilHochCallback( void );
void MenuPfeilRunterCallback( void );

// Die Anzahl der seit der letzten Sekunde vergangenen 10ms Schritte
unsigned char g_ZeitMs = 9;

// Die aktuelle Uhrzeit ist hier gespeichert
unsigned long g_Time = 0;

// Liste der Wochentage
unsigned char *Wochentage[] =
{
	"Mo",
	"Di",
	"Mi",
	"Do",
	"Fr",
	"Sa",
	"So"
};

typedef void (*KeyCallback)( void );

struct Taste
{
	unsigned char DownTime;
	unsigned char Delay1,Delay2;
	union
	{
		struct
		{
			bool Waiting:1;
			bool Pressed:1;
			bool Repeat:1;
		};
		char Status;
	};
	KeyCallback Ereignis;
} Tasten[6] =
{
	{0,2,1,{{0,0,0}},NULL},
	{0,2,1,{{0,0,0}},NULL},
	{0,2,1,{{0,0,0}},NULL},
	{0,2,1,{{0,0,0}},NULL},
	{0,2,1,{{0,0,0}},NULL},
	{0,2,1,{{0,0,0}},MenuEnableCallback},
};

struct Menu *g_ActMenu = NULL;

struct Menu Hauptmenue =
{
	false,
	0,3,
	{
	   {MenuEintrag, NULL,"NULL1"},
	   {MenuEintrag, NULL,"NULL2"},
	   {MenuEintrag, NULL,"NULL3"}
	}
};
// Alle 10ms ZeitMs erhhen
SIGNAL (SIG_OUTPUT_COMPARE1A)
{
	g_ZeitMs++;
	TCNT1 = 0;
}

// Bringt die LED und LCD Anzeige auf den neuesten Stand
void Display_Aktualisieren( void )
{
	// Temperatur messen
	signed char Temp = THERMO_GetTemp();
	//THERMO_Init_Temp_Read();
	
   // Zeit zerlegen
	struct Time Zeitpunkt;
	ZeitZerlegen(g_Time,&Zeitpunkt);
	
	// Auf dem LCD Display anzeigen
	if (g_ActMenu == NULL)
	{
	   char Zeit[32];
//	   sprintf(Zeit, " % 2d:%02d:%02d % 3dC\n %s, %02d.%02d.%04d" ,Zeitpunkt.Stunden,Zeitpunkt.Minuten, Zeitpunkt.Sekunden,Temp,
//					   Wochentage[Zeitpunkt.Wochentag],Zeitpunkt.Tag,Zeitpunkt.Monat,Zeitpunkt.Jahr +2000);
	   sprintf(Zeit, " % 2d:%02d:%02d %2XC\n %s, %02d.%02d.%04d" ,Zeitpunkt.Stunden,Zeitpunkt.Minuten, Zeitpunkt.Sekunden,Temp,
					   Wochentage[Zeitpunkt.Wochentag],Zeitpunkt.Tag,Zeitpunkt.Monat,Zeitpunkt.Jahr +2000);
	   LCD_ShowText(Zeit);
	};
		
	// Auf dem LED Display anzeigen
    LED_Init();
	LED_ShowTime(Zeitpunkt.Stunden, Zeitpunkt.Minuten);
};

unsigned char TimeDiff(unsigned char T1, unsigned char T2)
{
	return (T1 >= T2 ? T1 - T2 : 100 - T2 + T1);
};
// berprft den Status der angegebenen Taste und ruft eventuell dazugehrige
// Ereignisprozeduren auf
void CheckKey(unsigned char KeyNr)
{
	struct Taste *T = &Tasten[KeyNr];	
	if ((PINA & _BV(KeyNr)) == 0)
	{
		if (!T->Waiting)
		{
			T->Waiting = true;
			T->DownTime = g_ZeitMs;
		}
		else
		{
			if (!T->Pressed)
			{
				if (TimeDiff(g_ZeitMs, T->DownTime) > 1)
				{
					T->Pressed = true;
					T->DownTime = g_ZeitMs;
					if (T->Ereignis)
						T->Ereignis();
				};
			}
			else
			{
			   if (TimeDiff(g_ZeitMs, T->DownTime) > (!T->Repeat ? T->Delay1 : T->Delay2))
			   {
			   	T->Repeat = true;
					T->DownTime = g_ZeitMs;
					if (T->Ereignis)
						T->Ereignis();
			   };
			};			  		
		};
	}
	else
	{
		T->Status = 0;
		T->DownTime = 0;
	};
};

// Wird beim Bettigen der Men Taste aufgerufen
void MenuEnableCallback ( void )
{
	EnableMenu(&Hauptmenue);
};
// Wird beim Bettigen der Abbrechen Taste aufgerufen
void MenuDisableCallback ( void )
{
	DisableMenu();
};

// Aktiviert das Men
void EnableMenu(struct Menu* MenuPtr)
{
	g_ActMenu = MenuPtr;
	g_ActMenu->AktEintrag = 0;
	ShowMenu();
	Tasten[0].Ereignis = MenuPfeilHochCallback;
	Tasten[3].Ereignis = MenuPfeilRunterCallback;
	Tasten[4].Ereignis = MenuDisableCallback;
	Tasten[5].Ereignis = NULL;
};

// Deaktiviert das Men
void DisableMenu( void )
{
	g_ActMenu->AktEintrag = 0;
	g_ActMenu = NULL;
	Tasten[0].Ereignis = NULL;
	Tasten[3].Ereignis = NULL;
	Tasten[4].Ereignis = NULL;
	Tasten[5].Ereignis = MenuEnableCallback;
	Display_Aktualisieren();
};

// Bewegt sich im Men nach oben
void MenuPfeilHochCallback( void )
{
	if (g_ActMenu->AktEintrag == 0)
		g_ActMenu->AktEintrag = g_ActMenu->NrLines-1;
	else
		g_ActMenu->AktEintrag--;
	
	ShowMenu();
};

// Bewegt sich im Men nach unten
void MenuPfeilRunterCallback( void )
{
	g_ActMenu->AktEintrag++;
	if (g_ActMenu->AktEintrag ==  g_ActMenu->NrLines)
		g_ActMenu->AktEintrag = 0;
	
	ShowMenu();
};

// Zeigt einen Eintrag im Men an
void ShowMenu( void )
{
	unsigned char Text[32];
	unsigned char *TextPtr;
	memset(Text,0,32);
	if (g_ActMenu->Doppelt)
	{
		struct MenuItem *Item = &g_ActMenu->ItemList[g_ActMenu->AktEintrag];
		Item->DisplayFunc(Text,Item->Data);
	}
	else
	{
		struct MenuItem *Item = &g_ActMenu->ItemList[g_ActMenu->AktEintrag];
		if (g_ActMenu->AktEintrag == g_ActMenu->NrLines - 1 )
		{
			Item--;
			Text[0] = ' ';
		}
		else
			Text[0] = '>';
			
		Item->DisplayFunc(Text+1,Item->Data);
		strcat(Text,"\n");
		
		TextPtr = Text + strlen(Text);
		if (g_ActMenu->AktEintrag == g_ActMenu->NrLines - 1 )
			*TextPtr = '>';
		else
			*TextPtr = ' ';
			
		Item++;
		Item->DisplayFunc(TextPtr+1,Item->Data);
		
	};
	
   LCD_ShowText(Text);
};

// Zeigt einen einfachen Meneintrag an
void MenuEintrag( char* Text, void *Data)
{
   strcpy(Text,Data);
};

int main (void)
{
	// Port A als eingang konfigurieren
	DDRA = 0x00;
	PORTA = 0xFF;
	
	// Alle anderen Ports zurcksetzen
	DDRB = 0xFF;
	DDRC = 0xFF;
	DDRD = 0xFF;
	PORTB = 0x0;
	PORTC = 0x0;
	PORTD = 0x0;

	LCD_BacklightON(0xFF);
	LCD_BacklightOFF();
    LED_Init();
    LED_ShowTime(12,48);
    LCD_Init();
    //LCD_ShowText("Hello World!\nHow are you?");

	// Timerinterrupt alle 10 ms
	OCR1A = 20000;
	TCCR1A = 0;
	TCCR1B = _BV(CS11) ;
 	TIMSK |= _BV(OCIE1A);
	sei();

	Display_Aktualisieren();
	Radio_Enable(8870);
	Radio_SetVolume(0);
	//Radio_Disable();
	
	// Hauptschleife
    for (;;)
    {
		SPI_Disable();
   		Display_Aktualisieren();

		// Wenn eine Sekunde vergangen ist, Zeit weiterzhlen
    	if (g_ZeitMs >= 100)
    	{
    		g_ZeitMs-= 100;
    		g_Time++;
		};

		// berprfen, ob eine Taste bettigt wurde
/*      for (i = 0; i < 6; i++)
        CheckKey(i);*/
    };

    return (0);
}

