//
//  Radio.c
//
// Routinen zur Ansteuerung des Radios
//

#include "MyUtils.h"
#include "Radio.h"
#include "LCD.H"


#define START 0x08
#define R_RECV 0x40
#define DATA_SEND 0x28
#define R_SEND 0x18


// Sendet daten ber die I2C Schnittstelle
unsigned char Radio_SendData(unsigned char Adresse, unsigned char *Data, unsigned char count)
{
	/* initialize TWI clock: 100 kHz clock, TWPS = 0 => prescaler = 1 */
	/* has prescaler (mega128 & newer) */
	TWSR = 0;
	TWBR = (16000000UL / 100000UL - 16) / 2;

	// bertragung starten
	TWCR |= (1<<TWINT)|(1<<TWSTA)| (1<<TWEN);
	
	// Warten, bis zur bertragung bereit
	while (!(TWCR & (1<<TWINT)));

	// Bei Fehler abbrechen	
	if ((TWSR & 0xF8) != START) return false;
	
	// Adresse bermitteln
	TWDR = Adresse;
	TWCR = (1<<TWINT) | (1<<TWEN);
	
	// Warten bis bertragung fertig	
	while (!(TWCR & (1<<TWINT)));

	// Abbrechen, wenn Fehler aufgetreten
	if ((TWSR & 0xF8) != R_SEND)
		return false;

	// Daten senden
	while (count--)
	{
		TWDR = *(Data++);
		TWCR = (1<<TWINT) | (1<<TWEN);

		// Warten bis bertragung fertig	
   	while (!(TWCR & (1<<TWINT)));

		// Abbrechen, wenn Fehler aufgetreten
		if ((TWSR & 0xF8) != DATA_SEND)
			return false;

	};
	
	// Warten bis bertragung fertig	
	while (!(TWCR & (1<<TWINT)));

	// Bus freigeben
 	TWCR = (1<<TWINT) | (1<<TWSTA) | _BV(TWSTO);
	
	return true;
};

// Ermittelt den Status des Radios
unsigned char Radio_ReadStatus( void )
{
	unsigned char Data = 0xFF;
	
	/* initialize TWI clock: 100 kHz clock, TWPS = 0 => prescaler = 1 */
	/* has prescaler (mega128 & newer) */
	TWSR = 0;
	TWBR = (16000000UL / 100000UL - 16) / 2;

	// bertragung starten
	TWCR |= (1<<TWINT)|(1<<TWSTA)| (1<<TWEN);
	
	// Warten, bis zur bertragung bereit
	while (!(TWCR & (1<<TWINT)));

	// Bei Fehler abbrechen	
	if ((TWSR & 0xF8) != START) return 0xFF;
	
	// Adresse bermitteln
	TWDR = 0xC1;
	TWCR = (1<<TWINT) | (1<<TWEN);
	
	// Warten bis bertragung fertig	
	while (!(TWCR & (1<<TWINT)));

	// Abbrechen, wenn Fehler aufgetreten
	if ((TWSR & 0xF8) != R_RECV)
		return 0xFF;
		
   // Byte empfangen		
	TWCR = (1<<TWINT) | (1<<TWEN);
	
	// Warten bis bertragung fertig	
	while (!(TWCR & (1<<TWINT)));

	// Daten einlesen
	if ((TWSR & 0xF8) == 0x58)
	{
	   Data = TWDR;
	};

	// Bus freigeben
 	TWCR = (1<<TWINT) | (1<<TWSTA) | _BV(TWSTO);
	
	return Data;
};

// Schaltet das Radio ein
void Radio_Enable(unsigned short Frequenz)
{
	unsigned char CommandsTuner[] = { 0x08,0xAE,0x80,0x19,0xA0 };
	unsigned char CommandsDataB[] = { 0x00, 0x0E};
	unsigned char CommandsDataC[] = { 0x01, 0xD0};
	unsigned char CommandsDataE[] = { 0x02, 0x77};
	unsigned short div = Frequenz / 5 + (unsigned short)(1070 / 5);
    CommandsTuner[0] = (div>>8) & 0x7f;
    CommandsTuner[1] = div      & 0xff;

	Radio_SendData(0xC0,CommandsTuner,5);
	Radio_SendData(0x86,CommandsDataB,2);
	Radio_SendData(0x86,CommandsDataC,2);
	Radio_SendData(0x86,CommandsDataE,2);
};

// Schaltet das Radio aus
void Radio_Disable( void )
{
//	unsigned char CommandsTuner[] = { 0x08,0xAE,0x80,0x19,0xA0 };
	unsigned char CommandsDataB[] = { 0x00, 0x20};
	unsigned char CommandsDataC[] = { 0x01, 0x00};
	unsigned char CommandsDataE[] = { 0x02, 0x00};

//	Radio_SendData(0xC0,CommandsTuner,5);
	Radio_SendData(0x86,CommandsDataB,2);
	Radio_SendData(0x86,CommandsDataC,2);
	Radio_SendData(0x86,CommandsDataE,2);
	
};

// Stellt die Lautstrke des Radios ein
void Radio_SetVolume(unsigned char Volume)
{
	SPI_Enable();
	DDRC = 0xFF;
	PORTC |= _BV(PC2);
	SPI_Send(0);
	SPI_Send(0);
	SPI_Send(Volume);
	PORTC &= ~_BV(PC2);
	SPI_Disable();
};
